#include "predef.h" 
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <sim5270.h>
#include <cfinter.h>


extern "C" { 
void UserMain(void * pd);
extern const unsigned long CPU_CLOCK;
void SetIntc( long func, int vector, int level, int prio );
}

const char * AppName="bare_serial";



volatile uartstruct *BareUart;


volatile DWORD nCharCount;





OS_SEM SemDataRx;
char RxBuffer[1000];
volatile int char_put_index;
volatile int char_get_index;



INTERRUPT( BARE_ISR, 0x2400 )
{
while (BareUart->usr & 1)
	{//Process incomming chars */
	 
	 char c = ( BareUart->utb ); 
 

	 /* For this example we will store all the chars up to a \n in a circular buffer */
	 if (c!='\n')
	 {
		 
	 RxBuffer[char_put_index++]=c;
	 if (char_put_index==1000) char_put_index=0;
	 
	 if (char_put_index==char_get_index)
		{/* If we got here we overflowed our circular buffer and should handle the error */

		 char_get_index++;
		 if (char_get_index==1000) char_get_index=0;
		}
	 }
	 else
		 OSSemPost( &SemDataRx ); 
	}

}



void OpenBareSerial(int port, unsigned long baud)
{

OSSemInit(&SemDataRx,0);

BareUart = &sim.uarts[port];

unsigned char ub1 = 0;
unsigned char ub2 = 0x41;

/* Calculate baud rate */
DWORD hdiv = ( ( ( CPU_CLOCK / 32 ) / baud ) + 1 ) >> 1;
ub1 = ( hdiv >> 8 ) & 0xFF;
ub2 = ( hdiv & 0xFF );

BareUart->ucr =  0x2a; /*Reset Rx Transmit disable Rx disable */
BareUart->ucr =  0x3a;    /*Reset Tx Transmit disable Rx disable */
BareUart->ucr =  0x1a;/*Transmit disable Rx disable reset mode reg */

BareUart->ucr =  0x10;
BareUart->umr =  0x13; /* umr1 8 data bits no parity */
BareUart->umr =  0x27;  /* umr2 1 stop*/ 

BareUart->usr =  0xDD; /* Use the internal timer for Baud Rate Generation */
BareUart->uipcr = 0x00; 
BareUart->uisr =   0x02;  /*Enable Rx  interrupts */

BareUart->dur = ub1; /* Actualy set the baud rate we calculated */
BareUart->dlr = ub2;


switch (port) /* Set up the hardware ports */
{
case 0:
	SetIntc( ( unsigned long ) &BARE_ISR, 13, 3, 1 );
	sim.gpio.par_uart |= 0x000F;   // MCF5271 RM 12.3.1.5.7
    break;
case 1:
	SetIntc( ( unsigned long ) &BARE_ISR, 14, 3, 1 );
	sim.gpio.par_uart |= 0x0FF0;   // MCF5271 RM 12.3.1.5.7
	break;
case 2:
	SetIntc( ( unsigned long ) &BARE_ISR, 15, 3, 1 );
	sim.gpio.par_uart |= 0x3000;   // MCF5271 RM 12.3.1.5.7
	break;
}//End of switch


BareUart->ucr = 0x05;/*Enable Tx and Rx */ 
}





void NoIsrWriteChar(int port, char c)
{
 while ( ( sim.uarts[port].usr & 4 ) == 0 ); 
 sim.uarts[port].utb = ( unsigned char ) c;  
}


void NoIsrWriteString(int port, const char * cp)
{
while(*cp) NoIsrWriteChar(port,*cp++);
}



void UserMain(void * pd)
{
   InitializeStack();
   if (EthernetIP == 0) GetDHCPAddress(); 
   OSChangePrio(MAIN_PRIO);
   EnableAutoUpdate();

   OpenBareSerial(1,115200);


   iprintf("Application started\n");

   NoIsrWriteString(1,"We will dump chars when we receive a \\n\r\n"); 
   while (1)
   {  int put_index,get_index;

	   OSSemPend(&SemDataRx,0);	//Wait forever
	   /* We don't want to be interrupted while we get the indexes */
	   USER_ENTER_CRITICAL();
	   put_index=char_put_index; 
	   get_index=char_get_index; 
	   USER_EXIT_CRITICAL();
	   

	   /* Now to print out our circular buffer */
	   iprintf("We got a buffer [");
	   {
	   while (get_index!=put_index)
		{
		 iprintf("%c",RxBuffer[get_index++]);
		 if (get_index==1000) get_index=0;
		}
	   char_get_index=get_index; 
	   iprintf("]\r\n");

	   }
  }
}
